from django.shortcuts import render_to_response, get_object_or_404
from django.http import HttpResponseRedirect, HttpResponse
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from models import Project, Status, Category, Department
from forms import ProjectInitiationForm, ReportForm
from django.contrib.auth.models import User

##
# Change or create a project
##
@login_required
def change_project(request, id=None):
	if id is None:		# If we didn't specify a project in the URL, don't try to look one up
		instance = None
	else:
		instance = get_object_or_404(Project, id=id)	# Using the ID specified in the URL, try to get the corresponding Project object. Else, 404

		# If the instance is marked as inactive, or if the person trying to edit the project is not its originator, give them a message and redirect to readonly view
		message = None
		if not instance.is_active:
			message = "Sorry, project #%s has been marked as inactive and is therefore read-only." % id
		if not(instance.originator == request.user):
			message = "Sorry, project #%s can only be edited by %s." % (id, instance.originator.get_full_name())
		if message:
			request.user.message_set.create(message=message)
			return HttpResponseRedirect('/view/%s/' % id)

	if request.method == 'POST':	# If we've POSTed (i.e. Submitted) the form...
		projectform = ProjectInitiationForm(request.POST, instance=instance)	# Create an instance of the form, populated with the POST data, and an instance (if applicable)
		if projectform.is_valid():
			new_project = projectform.save(commit=False)	# If the form is valid, save it,  but don't commit yet, since we need to change some stuff
			try:	# Get the Status object whose name is 'New'...
				stat = Status.objects.get(name__iexact='New')
			except Status.DoesNotExist: # ...but if it doesn't exist, create it
				stat = Status.objects.create(
					status='New',
					description='This is a new project request.',
				)
			new_project.status = stat	# New project's status is "New"
			new_project.originator = request.user	# New project's originator is the currently-logged-in user
			new_project.save()	# Okay *now* we can save it for real
			request.user.message_set.create(message="Project #%s was saved successfully." % new_project.id)
			return HttpResponseRedirect('/view/%s/' % new_project.id)	# Redirect to the readonly view of the project we just saved
	else:	# If we're GETting the view, we haven't clicked "submit" yet
		projectform = ProjectInitiationForm(instance=instance)
	return render_to_response('change_project.html', locals(), context_instance=RequestContext(request))



##
# View a single project, or view multiple projects, optionally filtered by user
##
@login_required
def view_project(request, id=None, filteruser=None):
	if id:	# If we specified an id in the url, get that project
		instance = get_object_or_404(Project, id=id)
	else:	# If we didn't specify an id in the url, get a list of projects
		project_list = Project.objects.all()
		if filteruser:	# If we specified a username in the url, filter by it...
			filteruser = get_object_or_404(User, username=filteruser)	# ...but only if it's a real username
			project_list = project_list.filter(originator=filteruser)

		# If the user requested sorting, by clicking on one of the links in the header of the table, do the sorting
		list_of_fields = [ x.name for x in Project._meta.fields ]
		if (list_of_fields.__contains__(request.GET.get('orderby')) | (str(request.GET.get('orderby')).find('__') > 0)):
			if request.GET.__contains__('desc'):
				project_list = project_list.order_by('-'+request.GET.get('orderby'))
			else:
				project_list = project_list.order_by(request.GET.get('orderby'))
				asc = True
		else:	# If the user didn't ask for any special sorting, just sort by ID
			project_list = project_list.order_by('id')
	return render_to_response('view_project.html', locals(), context_instance=RequestContext(request))


##
# Generate Reports
##
@login_required
def report_gen(request):
	if 'bsubmit' in request.GET:		# If we clicked "generate report"
		reportform = ReportForm(request.GET)	# Instantiate a copy of the ReportForm with the GET data
		query = Project.objects.all()		# Create a really general query
		criteria = []				# Create a human-readable list of criteria to display in the template
		if reportform.is_valid():			# If the form passes validation tests, process it
			category = reportform.cleaned_data['category']	# None or Category object
			status = reportform.cleaned_data['status']	# None or Status object
			department = reportform.cleaned_data['department']	# None or Department object
			order_by = reportform.cleaned_data['order_by']	# String
			include_comments = reportform.cleaned_data['include_comments']	# Bool
			include_inactive_projects = reportform.cleaned_data['include_inactive_projects']	# Bool
			if category:	# Keep adding stuff to the query, depending on values in the form
				query = query.filter(category=category)
				criteria.append('Category: %s' % category)
			if status:
				query = query.filter(status=status)
				criteria.append('Status: %s' % status)
			if department:
				query = query.filter(department=department)
				criteria.append('Department: %s' % department)
			if order_by:
				query = query.order_by(order_by)
				criteria.append('Order by: %s' % order_by)
			if not include_inactive_projects:
				query = query.filter(is_active=True)
				criteria.append('Including inactive projects')
	else:
		form = ReportForm()	# If we haven't clicked "submit", show the form
	return render_to_response('report_gen.html', locals(), context_instance=RequestContext(request))
